/////////////////////////////////////////////////////////////////////////////
//  File:       BinTreeFormat.cpp
//  Version:    1.0.0.0
//  Created:    03-april-2002
//
//  Author:     Thorsten Wack
//  E-mail:     wt@umsicht.fhg.de
//
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. If 
// the source code in  this file is used in any commercial application 
// then a simple email would be nice.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability if it causes any damage whatsoever.
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "bintree.h"
#include "bintreeformat.h"

/////////////////////////////////////////////////////////////////////////////
// the Format Helper-Class
CBinTreeFormat theFormat;

CBinTreeFormat* GetFormat()
{
	return& theFormat;
}

/////////////////////////////////////////////////////////////////////////////
IMPLEMENT_SERIAL(CFormatInfoOb, CObject, 1)

CFormatInfoOb::CFormatInfoOb()
{
	m_strFaceName="Times New Roman";
	m_crColor=RGB(0,0,0);
	m_bBold=FALSE;
	m_bItalic=FALSE;
	m_lfCharSet=ANSI_CHARSET;
}

CFormatInfoOb::CFormatInfoOb(CString strFaceName, COLORREF crColor, BOOL bBold, BOOL bItalic, BYTE lfCharSet)
{	
	m_strFaceName=strFaceName;
	m_crColor=crColor;
	m_bBold=bBold;
	m_bItalic=bItalic;
	m_lfCharSet=lfCharSet;
}

const void CFormatInfoOb::operator=(const CFormatInfoOb& rFormatInfoOb)
{
	m_strFaceName=rFormatInfoOb.m_strFaceName;
	m_crColor=rFormatInfoOb.m_crColor;
	m_bBold=rFormatInfoOb.m_bBold;
	m_bItalic=rFormatInfoOb.m_bItalic;
	m_lfCharSet=rFormatInfoOb.m_lfCharSet;
}

void CFormatInfoOb::Serialize(CArchive& ar)
{
	if(ar.IsStoring())
	{
		ar << m_strFaceName;
		ar << m_crColor;
		ar << m_bBold;
		ar << m_bItalic;
		ar << m_lfCharSet;
	}
	else
	{
		ar >> m_strFaceName;
		ar >> m_crColor;
		ar >> m_bBold;
		ar >> m_bItalic;
		ar >> m_lfCharSet;
	}
}

/////////////////////////////////////////////////////////////////////////////

CBinTreeFormat::CBinTreeFormat()
{
// 	Standard
	m_pFormatStd=new CFormatInfoOb("Times New Roman",
						::GetSysColor(COLOR_WINDOWTEXT),
						FALSE,  // Bold
						FALSE,  // Italic
						ANSI_CHARSET); 

// 	Variable
	m_pFormatVar=new CFormatInfoOb("Times New Roman",
						RGB(0,0,128),//::GetSysColor(COLOR_WINDOWTEXT),
						FALSE,  // Bold
						TRUE,   // Italic
						ANSI_CHARSET);  

// 	Constant
	m_pFormatConst=new CFormatInfoOb("Times New Roman",
						RGB(128,0,0),//::GetSysColor(COLOR_WINDOWTEXT),
						FALSE,  // Bold
						TRUE,   // Italic
						ANSI_CHARSET);

// 	Number
	m_pFormatNum=new CFormatInfoOb("Times New Roman",
						RGB(255,0,0),//::GetSysColor(COLOR_WINDOWTEXT),
						FALSE,  // Bold
						FALSE,  // Italic
						ANSI_CHARSET);
	
// 	Function
	m_pFormatFunc=new CFormatInfoOb("Times New Roman",
						RGB(0,0,255),//::GetSysColor(COLOR_WINDOWTEXT),
						TRUE,   // Bold
						FALSE,  // Italic
						ANSI_CHARSET);

// 	Operator
	m_pFormatOp=new CFormatInfoOb("Symbol",
						::GetSysColor(COLOR_WINDOWTEXT),
						FALSE,  // Bold
						FALSE,  // Italic
						SYMBOL_CHARSET);

// 	Symbol
	m_pFormatSym=new CFormatInfoOb("Symbol",
						::GetSysColor(COLOR_WINDOWTEXT),
						FALSE,  // Bold
						FALSE,  // Italic
						SYMBOL_CHARSET);

// 	Physical Constant
	m_pFormatPhys=new CFormatInfoOb("MT Extra",
						::GetSysColor(COLOR_WINDOWTEXT),
						FALSE,  // Bold
						FALSE,  // Italic
						SYMBOL_CHARSET);


//Standard
	m_nFontHeight[FH_STANDARD]=12;
//Subscript/Superscript
	m_nFontHeight[FH_SUB]=10;
//Sub-Subscript/Superscript
	m_nFontHeight[FH_SUB_SUB]=8;
//Symbol
	m_nFontHeight[FH_SYMBOL]=16;
//Sub-Symbol
	m_nFontHeight[FH_SUB_SYMBOL]=12;

	m_FormatMap[NT_VARIABLE]	=m_pFormatVar;	// variable like m(mass)...
	m_FormatMap[NT_CONSTANT]	=m_pFormatConst;// Constant like pi, c...
	m_FormatMap[NT_FUNC]		=m_pFormatFunc;	// like sin, cos, tan
	m_FormatMap[NT_USERFUNC]	=m_pFormatFunc;	// f(x)
	m_FormatMap[NT_EXTFUNC]		=m_pFormatFunc;	// external functions
	m_FormatMap[NT_NUMBER]		=m_pFormatNum;	// 3,1 and so on

	m_FormatMap[NT_STANDARD]	=m_pFormatStd;	// standard-node
	m_FormatMap[NT_DIVISION]	=m_pFormatOp;	// x over y
	m_FormatMap[NT_NTHROOT]		=m_pFormatSym;	// n-th root of anything
	m_FormatMap[NT_ROOT]		=m_pFormatSym;  // squareroot
	m_FormatMap[NT_POWERTO]		=m_pFormatOp;	// x powered by y
	m_FormatMap[NT_PLACEHOLDER]	=m_pFormatStd;	// placeholder for nodes
	m_FormatMap[NT_BRACE]		=m_pFormatSym;	// Brackes {x+y}
	m_FormatMap[NT_PLUS]		=m_pFormatOp;	// x + y
	m_FormatMap[NT_MINUS]		=m_pFormatOp;	// x - y
	m_FormatMap[NT_TIMES]		=m_pFormatOp;	// x * y
	m_FormatMap[NT_EQUATION]	=m_pFormatOp;	// x = y
	m_FormatMap[NT_PARTDERIVE]	=m_pFormatOp;	// part. d/dt
	m_FormatMap[NT_DERIVE]		=m_pFormatStd;	// d/dt fkt(t)
	m_FormatMap[NT_SUM]			=m_pFormatSym;	// Sum from i=0;to i=10, over fkt(i)
	m_FormatMap[NT_PROD]		=m_pFormatSym;	// Product from i=0;to i=10, over fkt(i)
	m_FormatMap[NT_RANGE]		=m_pFormatOp;	// Upper and lower range
	m_FormatMap[NT_INTEGRAL]	=m_pFormatSym;	// Integral from i=0;to i=10, over fkt(t) dt
	m_FormatMap[NT_INTEGRAND]	=m_pFormatStd;	// Integrandvariable and function
	m_FormatMap[NT_POISON]		=m_pFormatSym;	// like (a,b) {a,b}
	m_FormatMap[NT_INDEX]		=m_pFormatStd;	// Subscript Node
	m_FormatMap[NT_VECTOR]		=m_pFormatSym;	// Overline like vector...
	m_FormatMap[NT_LIMES]		=m_pFormatFunc;	// lim(x->oo) f(x)
	m_FormatMap[NT_OPERATOR]	=m_pFormatOp;	// x->oo
	m_FormatMap[NT_BINOMIAL]	=m_pFormatFunc;	// n over k
	m_FormatMap[NT_MATRIX]		=m_pFormatStd;	// Matrixnode
	m_FormatMap[NT_LINE]		=m_pFormatStd;	// line in a matrix
	m_FormatMap[NT_ELEMENT]		=m_pFormatStd;	// Elementnode in a matrix
	m_FormatMap[NT_NABLA]		=m_pFormatSym;	// Nablaoperator
	m_FormatMap[NT_LAPLACE]		=m_pFormatSym;	// Laplaceoperator
	m_FormatMap[NT_CROSS]		=m_pFormatOp;	// Crossproduct of a vector
	m_FormatMap[NT_ARROWS]		=m_pFormatSym;	// ->, =>
	m_FormatMap[NT_INFINITY]	=m_pFormatSym;	// Infinity oo
	m_FormatMap[NT_PLANCK]		=m_pFormatPhys;	// Planck h
	m_FormatMap[NT_LAMBDA]		=m_pFormatPhys;	// Lambda
	m_FormatMap[NT_LABEL]		=m_pFormatStd;	// Lambda
}

CBinTreeFormat::~CBinTreeFormat()
{
	delete m_pFormatStd;
	delete m_pFormatVar;
	delete m_pFormatConst;
	delete m_pFormatNum;
	delete m_pFormatFunc;
	delete m_pFormatOp;
	delete m_pFormatSym;
	delete m_pFormatPhys;
}

int CBinTreeFormat::GetFontHeight(int nIndex)
{
	ASSERT(nIndex < (sizeof(m_nFontHeight)/sizeof(int)) && nIndex>=0);
	return m_nFontHeight[nIndex];
}

const void CBinTreeFormat::operator=(const CBinTreeFormat& rBinTreeFormat)
{
	(*m_pFormatStd) = *(rBinTreeFormat.m_pFormatStd);
	(*m_pFormatVar) = *(rBinTreeFormat.m_pFormatVar);
	(*m_pFormatConst) = *(rBinTreeFormat.m_pFormatConst);
	(*m_pFormatNum) = *(rBinTreeFormat.m_pFormatNum);
	(*m_pFormatFunc) = *(rBinTreeFormat.m_pFormatFunc);
	(*m_pFormatOp) = *(rBinTreeFormat.m_pFormatOp);
	(*m_pFormatSym) = *(rBinTreeFormat.m_pFormatSym);
	(*m_pFormatPhys) = *(rBinTreeFormat.m_pFormatPhys);

	memcpy( &(m_nFontHeight[0]), &(rBinTreeFormat.m_nFontHeight[0]), sizeof(m_nFontHeight));
}

void CBinTreeFormat::Serialize(CArchive& ar)
{

	m_pFormatStd->Serialize(ar);
	m_pFormatVar->Serialize(ar);
	m_pFormatConst->Serialize(ar);
	m_pFormatNum->Serialize(ar);
	m_pFormatFunc->Serialize(ar);
	m_pFormatOp->Serialize(ar);
	m_pFormatSym->Serialize(ar);
	if(ar.IsStoring())
	{
		ar << m_nFontHeight[0];
		ar << m_nFontHeight[1];
		ar << m_nFontHeight[2];
		ar << m_nFontHeight[3];
		ar << m_nFontHeight[4];
	}
	else
	{
		ar >> m_nFontHeight[0];
		ar >> m_nFontHeight[1];
		ar >> m_nFontHeight[2];
		ar >> m_nFontHeight[3];
		ar >> m_nFontHeight[4];
	}
}

