/////////////////////////////////////////////////////////////////////////////
//  File:       FormulaCtrl.cpp
//  Version:    1.0.0.0
//  Created:    03-april-2002
//
//  Author:     Thorsten Wack
//  E-mail:     wt@umsicht.fhg.de
//
//  parts of Code by Chris Maunder
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. If 
// the source code in  this file is used in any commercial application 
// then a simple email would be nice.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability if it causes any damage whatsoever.
//
// Version 1.0.0.1: OnKeyDown was modified. Pressing a key on a selected 
//					placeholder adds a new node. suggested by John A. Johnson
//					OnEraseBkgnd changed (now flickerfree)
//					OnPaint changed (now flickerfree)
//					Modfied drag&drop 
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "resource.h"
#include "formulactrl.h"
#include "formulainplaceedit.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(CFormulaCtrl, CWnd)

// by Chris Maunder from GridCtrl
// Get the number of lines to scroll with each mouse wheel notch
// Why doesn't windows give us this function???
UINT GetMouseScrollLines()
{
    int nScrollLines = 3;            // reasonable default

#ifndef _WIN32_WCE
    // Do things the hard way in win95
    OSVERSIONINFO VersionInfo;
    VersionInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
    if (!GetVersionEx(&VersionInfo) || 
        (VersionInfo.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS && VersionInfo.dwMinorVersion == 0))
    {
        HKEY hKey;
        if (RegOpenKeyEx(HKEY_CURRENT_USER,  _T("Control Panel\\Desktop"),
            0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
        {
            TCHAR szData[128];
            DWORD dwKeyDataType;
            DWORD dwDataBufSize = sizeof(szData);
            
            if (RegQueryValueEx(hKey, _T("WheelScrollLines"), NULL, &dwKeyDataType,
                (LPBYTE) &szData, &dwDataBufSize) == ERROR_SUCCESS)
            {
                nScrollLines = _tcstoul(szData, NULL, 10);
            }
            RegCloseKey(hKey);
        }
    }
    // win98 or greater
    else
           SystemParametersInfo (SPI_GETWHEELSCROLLLINES, 0, &nScrollLines, 0);
#endif

    return nScrollLines;
}

/////////////////////////////////////////////////////////////////////////////
// CFormulaCtrl
CFormulaCtrl::CFormulaCtrl()
{
	RegisterWindowClass();

    _AFX_THREAD_STATE* pState = AfxGetThreadState();
    if (!pState->m_bNeedTerm && !AfxOleInit())
        AfxMessageBox(_T("OLE initialization failed. Make sure that the OLE libraries are the correct version"));
	
	CF_BINTREE=RegisterClipboardFormat(FORMULACTRL_CLASSNAME);
	
	m_pFocusNode=NULL;

    m_nBarState = FCSB_NONE;
    m_nVScrollMax         = 0;          // Scroll position
    m_nHScrollMax         = 0;
	m_nRowsPerWheelNotch  = GetMouseScrollLines();
	m_pEditWnd	=NULL;
	m_VirtualRect=CRect(0,0,0,0);
	
	m_bPreparedToDrag=FALSE;
	m_bPreparedToEdit=FALSE;
	m_bReadOnly=FALSE;

	ASSERT(m_pBinTree);
}

CFormulaCtrl::~CFormulaCtrl()
{
    COleDataSource *pSource = COleDataSource::GetClipboardOwner();
    if(pSource)
        COleDataSource::FlushClipboard();
}

void CFormulaCtrl::SetBinTree(CBinTree* pBinTree)
{
	m_pBinTree=pBinTree;
}

CBinTree* CFormulaCtrl::GetBinTree()
{
	return m_pBinTree;
}

// Register the window class if it has not already been registered.
BOOL CFormulaCtrl::RegisterWindowClass()
{
    WNDCLASS wndcls;
    HINSTANCE hInst = AfxGetInstanceHandle();

    if (!(::GetClassInfo(hInst, FORMULACTRL_CLASSNAME, &wndcls)))
    {
        // otherwise we need to register a new class
        wndcls.style            = CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
        wndcls.lpfnWndProc      = ::DefWindowProc;
        wndcls.cbClsExtra       = wndcls.cbWndExtra = 0;
        wndcls.hInstance        = hInst;
        wndcls.hIcon            = NULL;
        wndcls.hCursor          = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
        wndcls.hbrBackground    = ::GetSysColorBrush(COLOR_WINDOW);
        wndcls.lpszMenuName     = NULL;
        wndcls.lpszClassName    = FORMULACTRL_CLASSNAME;

        if (!AfxRegisterClass(&wndcls))
        {
            AfxThrowResourceException();
            return FALSE;
        }
    }

    return TRUE;
}

BEGIN_MESSAGE_MAP(CFormulaCtrl, CWnd)
	//{{AFX_MSG_MAP(CFormulaCtrl)
	ON_WM_PAINT()
    ON_WM_HSCROLL()
    ON_WM_VSCROLL()
	ON_WM_ERASEBKGND()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_KEYDOWN()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_SIZE()
	ON_WM_MOUSEWHEEL()
	ON_WM_RBUTTONUP()
	//}}AFX_MSG_MAP
    ON_NOTIFY(LVN_ENDLABELEDIT, IDC_INPLACE_CONTROL, OnEndInPlaceEdit)

    ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
    ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
    ON_COMMAND(ID_EDIT_CUT, OnEditCut)
    ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, OnUpdateEditCut)
    ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
    ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)

END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CFormulaCtrl message handlers
void CFormulaCtrl::OnDraw(CDC* pDC)
{
	CRect rect;
	GetClientRect(rect);

	m_pFocusNode=NULL;
	ResetScrollBars();
	m_pBinTree->DrawTree(pDC, rect);
}

void CFormulaCtrl::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
    CBitmap  bitmap;      // Offscreen bitmap
	CBitmap* pOldBitmap;
	CRect rect;
	CRect clrect;
	CRect destrect;
    CDC MemDC;
    
	MemDC.CreateCompatibleDC(&dc);
	MemDC.FillSolidRect(rect, ::GetSysColor(COLOR_WINDOW));

	rect=m_pBinTree->GetRect(&dc);
	m_VirtualRect=rect;	
    GetClientRect(clrect);
	destrect=clrect | rect;

	bitmap.CreateCompatibleBitmap(&dc, destrect.Width(), destrect.Height());
 	pOldBitmap = MemDC.SelectObject(&bitmap);

	MemDC.SetWindowOrg(0,0);
	MemDC.FillSolidRect(destrect, ::GetSysColor(COLOR_WINDOW));

    OnDraw(&MemDC);

	dc.BitBlt(rect.left, 
		rect.top, 
		clrect.Width(), 
		clrect.Height(),
		&MemDC, 
		GetScrollPos32(SB_HORZ),
		GetScrollPos32(SB_VERT),
		SRCCOPY);

	// Swap back the original bitmap.
	MemDC.SelectObject(pOldBitmap);
    if(m_pEditWnd)
		m_pEditWnd->RepositionWindow(&TransformRect(m_pBinTree->GetSelectedNode()->m_NodeRect));

	// Do not call CWnd::OnPaint() for painting messages
}

BOOL CFormulaCtrl::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE;//CWnd::OnEraseBkgnd(pDC);
}

void CFormulaCtrl::PreSubclassWindow() 
{
	// TODO: Add your specialized code here and/or call the base class
	// In our case this is not needed - yet - so just drop through to
    // the base class
	CWnd::PreSubclassWindow();
}

////////////////////////////////////////////////////////////////////////////
// CFormulaCtrl methods

BOOL CFormulaCtrl::Create(CWnd* pParentWnd, const RECT& rect, UINT nID, DWORD dwStyle /*=WS_VISIBLE*/)
{
	return CWnd::Create(FORMULACTRL_CLASSNAME, _T(""), dwStyle, rect, pParentWnd, nID);
}

void CFormulaCtrl::SetReadOnly(BOOL bReadOnly)
{
	m_bReadOnly=bReadOnly;
	Invalidate();
}

void CFormulaCtrl::TransformPoint(CPoint& point)
{
	point.x+=GetScrollPos32(SB_HORZ);
	point.y+=GetScrollPos32(SB_VERT);
}

CRect CFormulaCtrl::TransformRect(CRect& rect)
{
	CRect resRect(rect);
	resRect.OffsetRect(-GetScrollPos32(SB_HORZ), -GetScrollPos32(SB_VERT));
	return resRect;
}

void CFormulaCtrl::OnMouseMove(UINT nFlags, CPoint point) 
{
	CWnd::OnMouseMove(nFlags, point);
	if(m_bReadOnly)
		return;

	if(!m_pEditWnd)
	{
		TransformPoint(point);

		CClientDC dc(this); // device context for painting
		CRect resultRect(0,0,0,0);

		CNode* pNode=m_pBinTree->GetNodeFromPoint(&dc, resultRect, point);
		if(pNode==m_pFocusNode)
			return;

		if(m_pFocusNode)
			dc.DrawFocusRect(TransformRect(m_pFocusNode->m_NodeRect));
		
		if(pNode)
			dc.DrawFocusRect(TransformRect(pNode->m_NodeRect));

		m_pFocusNode=pNode;
		// perform Drag and drop only if left mouse button is down and
		// control is prepared to drag(selected leaf or selected subtree) 
		// setup of m_bPreparedToDrag takes place in OnLButtonDown and
		// OnLButtonDblClk
		if(nFlags & MK_LBUTTON && 
			m_bPreparedToDrag)
			OnBeginDrag();
	}
}

void CFormulaCtrl::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// get the selected node and set the selection type to NS_NODE
	// (select only the node))
	CWnd::OnLButtonDown(nFlags, point);

	if(m_bReadOnly)
		return;
	
	TransformPoint(point);
	SetFocus();

	CClientDC dc(this); // device context for painting
	CRect resultRect(0,0,0,0);

	CNode* pNode=m_pBinTree->GetNodeFromPoint(&dc, resultRect, point);
	if(pNode)
	{
		//Setup the m_bPreparedToEdit-member
		if(pNode==m_pBinTree->GetSelectedNode())
			m_bPreparedToEdit=TRUE;

		// if node with subtree is selcted, 
		// wait if drag action is performed
		if(m_bPreparedToDrag && pNode==m_pBinTree->GetSelectedNode())
			return;

		// Select the node
		m_pBinTree->SelectNode(pNode, NS_NODE);
		
		//Setup the m_bPreparedToDrag-member
		m_bPreparedToDrag=pNode->IsNodeLeaf();

		Invalidate(FALSE);
	}
}

void CFormulaCtrl::OnLButtonUp(UINT nFlags, CPoint point) 
{
	CWnd::OnLButtonUp(nFlags, point);

	if(m_bReadOnly)
		return;
	
	TransformPoint(point);
	SetFocus();
	
	CClientDC dc(this); // device context for painting
	CRect resultRect(0,0,0,0);
	CNode* pNode=m_pBinTree->GetNodeFromPoint(&dc, resultRect, point);
	if(pNode)
	{
		if(pNode==m_pBinTree->GetSelectedNode() && 
			(pNode->GetEditMode() & NE_ALLOWEDIT) &&
			m_bPreparedToEdit)
		{
			pNode->SetEditMode(pNode->GetEditMode() | NE_INEDITMODE);
			DWORD dwStyle = ES_CENTER;
			CRect rectEdit=TransformRect(pNode->m_NodeRect);

			m_pEditWnd = new CFormulaInPlaceEdit(
				this, 
				rectEdit,
				dwStyle, 
				IDC_INPLACE_CONTROL, 
				pNode,
				pNode->GetShortCut(), 
				VK_END);
			m_bPreparedToEdit=FALSE;
		}
		Invalidate(FALSE);
	}	
}


void CFormulaCtrl::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	// get the selected node and set the selection type to NS_SUBTREE
	// (select all children of the selected node))
	CWnd::OnLButtonDblClk(nFlags, point);

	if(m_bReadOnly)
		return;
	
	TransformPoint(point);
	SetFocus();

	CRect rect;
	GetClientRect(rect);
	rect.DeflateRect(2,2);

	CClientDC dc(this); // device context for painting
	CRect resultRect(0,0,0,0);

	CNode* pNode=m_pBinTree->GetNodeFromPoint(&dc, resultRect, point);
	m_pBinTree->SelectNode(pNode, NS_SUBTREE);
	m_bPreparedToDrag=TRUE;

	Invalidate(FALSE);
}

void CFormulaCtrl::OnRButtonUp(UINT nFlags, CPoint point) 
{
}

////////////////////////////////////////////////////////////////////////////////////////
// Clipboard functions

// Deletes selected subtree
void CFormulaCtrl::CutSubTree()
{
    if (m_bReadOnly)
        return;

    CNode* pNode=GetBinTree()->GetSelectedNode();

    if(pNode)
	{
		GetBinTree()->ReplaceSubTree(pNode, NT_PLACEHOLDER);

		// Reformat nodes to make sure the fontheights are correct
		// (in case a node with normal-font-height is pasted into a subscript)
		GetBinTree()->ReformatNodes();
		Invalidate();
	}
}

// Copies selected subtree into the clipboard
COleDataSource* CFormulaCtrl::CopySubTree()
{
	CNode* pNode=GetBinTree()->GetSelectedNode();
	if(pNode==NULL)
		return NULL;
	
	// Write to shared file
	CSharedFile sf(GMEM_MOVEABLE | GMEM_DDESHARE | GMEM_ZEROINIT);

	CArchive ar (&sf, CArchive::store);
	CBinTree tempTree;
	// Build a BinTree with pNode as root
	tempTree.SetRootNode(pNode);
	// Serialize it
	tempTree.Serialize(ar);
	// Set root node to Null so the BinTree-destructor doesnt affect the nodes
	tempTree.SetRootNode(NULL);
	ar.Close();
	
	DWORD dwLen = (DWORD) sf.GetLength();
	HGLOBAL hMem = sf.Detach();
	if (!hMem)
		return NULL;

	hMem = ::GlobalReAlloc(hMem, dwLen, GMEM_MOVEABLE | GMEM_DDESHARE | GMEM_ZEROINIT);
	if (!hMem)
		return NULL;

	// Cache data
	COleDataSource* pSource = new COleDataSource();
	pSource->CacheGlobalData(CF_BINTREE, hMem);

	return pSource;
}

// Pastes subtree from the clipboard into the selected node
BOOL CFormulaCtrl::PasteSubTree(CNode* pNode, COleDataObject* pDataObject)
{
	CNode* pParentNode=GetBinTree()->GetParent(pNode);
	// Get the text from the COleDataObject
	HGLOBAL hmem = pDataObject->GetGlobalData(CF_BINTREE);
	if(!hmem)
		return FALSE;

	CMemFile sf((BYTE*) ::GlobalLock(hmem), ::GlobalSize(hmem));

	CArchive ar (&sf, CArchive::load);
	CBinTree tempTree;
	tempTree.Serialize(ar);
	
	GetBinTree()->ReplaceNode(pNode, tempTree.GetRootNode());
	// Set root node to Null so the BinTree-destructor doesnt affect the nodes
	tempTree.SetRootNode(NULL);
	ar.Close();
	
	::GlobalUnlock(hmem);
	// Reformat nodes to make sure the fontheights are correct
	// (in case a node with normal-font-height is pasted into a subscript)
	GetBinTree()->ReformatNodes();

	Invalidate();
    
	return TRUE;
}

void CFormulaCtrl::OnEditCut()
{
    if (m_bReadOnly)
        return;

	// copy the selected subtree
	COleDataSource* pSource = CopySubTree();
    if (!pSource)
        return;

    pSource->SetClipboard();
	// delete the selected subtree
    CutSubTree();
	GetBinTree()->CheckPoint();
}

void CFormulaCtrl::OnEditCopy()
{
	// copy the selected subtree
    COleDataSource* pSource = CopySubTree();
    if (!pSource)
        return;

    pSource->SetClipboard();
}

void CFormulaCtrl::OnEditPaste()
{
	CNode* pNode=GetBinTree()->GetSelectedNode(); 
	if(pNode==NULL)
		return;
	
	// only paste, if the selected node is of type NT_PLACEHOLDER
	if(pNode->GetNodeType()!=NT_PLACEHOLDER)
		return;
    // Attach a COleDataObject to the clipboard and paste the data to the formula control
    COleDataObject obj;
    if (obj.AttachClipboard())
    {
		PasteSubTree(pNode, &obj);
		// set checkpoint for undo/reso
		GetBinTree()->CheckPoint();
	}
}

void CFormulaCtrl::OnEditSelectAll()
{
	CNode* pNode=GetBinTree()->GetRootNode();
	// if tree has a root node then select it in NS_SUBTREE mode
	if(pNode)
	{
		GetBinTree()->SelectNode(pNode, NS_SUBTREE);
		Invalidate();
	}
}

void CFormulaCtrl::OnUpdateEditCopy(CCmdUI* pCmdUI)
{
}

void CFormulaCtrl::OnUpdateEditCut(CCmdUI* pCmdUI)
{
}

void CFormulaCtrl::OnUpdateEditPaste(CCmdUI* pCmdUI)
{
}

void CFormulaCtrl::OnUpdateEditSelectAll(CCmdUI* pCmdUI)
{
}

// Start drag and drop
void CFormulaCtrl::OnBeginDrag()
{
    COleDataSource* pSource=CopySubTree();
    if (pSource)
    {
        CNode* pNode=GetBinTree()->GetSelectedNode();
		DROPEFFECT dropEffect=pSource->DoDragDrop(DROPEFFECT_COPY | DROPEFFECT_MOVE);
        GetBinTree()->SelectNode(pNode, NS_SUBTREE);

        if (dropEffect & DROPEFFECT_MOVE)
            CutSubTree();

        if (pSource)
            delete pSource;    // Did not pass source to clipboard, so must delete
    }
}

// Handle drag over formula-control
DROPEFFECT CFormulaCtrl::OnDragOver(COleDataObject* pDataObject, DWORD dwKeyState,
                                 CPoint point)
{
	// Any data available for us?
    if (m_bReadOnly || !pDataObject->IsDataAvailable(CF_BINTREE))
        return DROPEFFECT_NONE;

	TransformPoint(point);

	CClientDC dc(this); // device context for painting
	CRect resultRect(0,0,0,0);

	CNode* pNode=m_pBinTree->GetNodeFromPoint(&dc, resultRect, point);
	if(pNode!=m_pFocusNode)
	{
		if(m_pFocusNode)
			dc.DrawFocusRect(TransformRect(m_pFocusNode->m_NodeRect));
		if(pNode)
			dc.DrawFocusRect(TransformRect(pNode->m_NodeRect));
		m_pFocusNode=pNode;
	}
	if(m_pFocusNode==NULL)
        return DROPEFFECT_NONE;

	// make things easier
	// if node is not a placeholder then nothing happens
	if(m_pFocusNode->GetNodeType()!=NT_PLACEHOLDER)
        return DROPEFFECT_NONE;

    // Return an appropraite value of DROPEFFECT so mouse cursor is set properly
    if (dwKeyState & MK_CONTROL)
        return DROPEFFECT_COPY;
    else
        return DROPEFFECT_MOVE;
}

// Something has just been dragged onto the formula-control
DROPEFFECT CFormulaCtrl::OnDragEnter(COleDataObject* pDataObject, DWORD dwKeyState,
                                  CPoint point)
{
    // Any data available for us?
    if (pDataObject->IsDataAvailable(CF_BINTREE))
        return DROPEFFECT_NONE;

	TransformPoint(point);

	CClientDC dc(this); // device context for painting
	CRect resultRect(0,0,0,0);

	CNode* pNode=m_pBinTree->GetNodeFromPoint(&dc, resultRect, point);
	if(pNode!=m_pFocusNode)
	{
		if(m_pFocusNode)
			dc.DrawFocusRect(TransformRect(m_pFocusNode->m_NodeRect));
		if(pNode)
			dc.DrawFocusRect(TransformRect(pNode->m_NodeRect));
		m_pFocusNode=pNode;
	}
	if(m_pFocusNode==NULL)
        return DROPEFFECT_NONE;

	// make things easier
	// if node is not a placeholder then nothing happens
	if(m_pFocusNode->GetNodeType()!=NT_PLACEHOLDER)
        return DROPEFFECT_NONE;

    // Return an appropraite value of DROPEFFECT so mouse cursor is set properly
    if (dwKeyState & MK_CONTROL)
        return DROPEFFECT_COPY;
    else
        return DROPEFFECT_MOVE;
}

// Something has just been dragged away from the formula-control
void CFormulaCtrl::OnDragLeave()
{
}

// Something has just been dropped onto the formula-control
BOOL CFormulaCtrl::OnDrop(COleDataObject* pDataObject, DROPEFFECT /*dropEffect*/,
                       CPoint /* point */)
{
	if(m_pFocusNode==NULL)
		return FALSE;
	// make things easier
	// if node is not a placeholder then nothing happens
	if(m_pFocusNode->GetNodeType()!=NT_PLACEHOLDER)
		return FALSE;

    OnDragLeave();

    BOOL bRet=PasteSubTree(m_pFocusNode, pDataObject);
	// if operation was successful, 
	// set checkpoint for undo/reso
	if(bRet)
		GetBinTree()->CheckPoint();

	return bRet;

}

void CFormulaCtrl::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if(!m_bReadOnly)
	{
		if (IsCTRLpressed())
		{
			switch (nChar)
			{
			case 'A':
				OnEditSelectAll();
				break;
			case 'X':
				OnEditCut();
				break;
			case VK_INSERT:
			case 'C':
				OnEditCopy();
				break;
			case 'V':
				OnEditPaste();
				break;
			}
			return;
		}

		if (IsSHIFTpressed() &&(nChar == VK_INSERT))
			OnEditPaste();
		
		if (nChar == VK_RIGHT)
		{
			if(m_pBinTree->GetSelectedNode())
				if(m_pBinTree->GetSelectedNode()->GetRightChild())
				{
					m_pBinTree->SelectNode(
						m_pBinTree->GetSelectedNode()->GetRightChild(),
						m_pBinTree->GetNodeSelectionType());
					Invalidate();
				}
		}
		if (nChar == VK_DOWN)
		{
		}
		if (nChar == VK_LEFT)
		{
			if(m_pBinTree->GetSelectedNode())
				if(m_pBinTree->GetSelectedNode()->GetLeftChild())
				{
					m_pBinTree->SelectNode(
						m_pBinTree->GetSelectedNode()->GetLeftChild(),
						m_pBinTree->GetNodeSelectionType());
					Invalidate();
				}
		}
		if (nChar == VK_UP)
		{
			if(m_pBinTree->GetSelectedNode())
			{
				CNode* pParent=m_pBinTree->GetParent(m_pBinTree->GetSelectedNode());
				if(pParent)
				{
					m_pBinTree->SelectNode(
						pParent,
						m_pBinTree->GetNodeSelectionType());
					Invalidate();
				}
			}
		}

		if (nChar == VK_DELETE)
		{
			if(m_pBinTree->GetSelectedNode())
			{
				if(m_pBinTree->GetSelectedNode()!=m_pBinTree->GetRootNode() &&
					(m_pBinTree->GetNodeSelectionType()==NS_SUBTREE ||
					m_pBinTree->GetSelectedNode()->IsNodeLeaf()) &&
					(m_pBinTree->GetSelectedNode()->GetEditMode() & NE_ALLOWDELETE))
				{
					m_pBinTree->ReplaceSubTree(m_pBinTree->GetSelectedNode(), NT_PLACEHOLDER);
					// Send Notification to parent
					SendMessageToParent(LVN_ENDLABELEDIT);

					Invalidate();
				}
			}
		}

		if (nChar == VK_SPACE)
		{
			if(m_pBinTree->GetSelectedNode())
			{
				if(m_pBinTree->GetSelectedNode())
				{
					m_pBinTree->SetNodeSelectionType(m_pBinTree->GetNodeSelectionType()%2+1);
					Invalidate();
				}
			}
		}
	}
	if (isalnum(nChar))
	{
		CWnd::OnKeyDown(nChar, nRepCnt, nFlags);
		CNode* pNode=m_pBinTree->GetSelectedNode();
		// if node is allowed to be edited
		// perform the edit action
		if(pNode)
		{
			if(pNode->GetNodeType()==NT_PLACEHOLDER)
			{
				pNode=GetBinTree()->ReplaceNode(pNode, NT_VARIABLE);
				GetBinTree()->ReformatNodes();
				// force repainting (implict calculation of the new destination rect)
				SendMessage(WM_PAINT);
			}
			if(pNode->GetEditMode() & NE_ALLOWEDIT)
			{
				pNode->SetEditMode(pNode->GetEditMode() | NE_INEDITMODE);
				DWORD dwStyle = ES_CENTER;
				CRect rectEdit=TransformRect(pNode->m_NodeRect);

				m_pEditWnd = new CFormulaInPlaceEdit(
					this, 
					rectEdit,
					dwStyle, 
					IDC_INPLACE_CONTROL, 
					pNode,
					pNode->GetShortCut(), 
					nChar);
			}
		}
		return;
	}

	CWnd::OnKeyDown(nChar, nRepCnt, nFlags);
}

BOOL CFormulaCtrl::PreTranslateMessage(MSG* pMsg) 
{
	if ( pMsg->message == WM_KEYDOWN )
		switch ( (int)pMsg->wParam )
		{
			case VK_RIGHT:
			case VK_LEFT:
			case VK_UP:
			case VK_DOWN:
			case VK_DELETE:
				// dipatch keyboard messages directly (skip main accelerators)
				::TranslateMessage(pMsg);
				::DispatchMessage(pMsg);
				return TRUE;
		}
	
	return CWnd::PreTranslateMessage(pMsg);
}

void CFormulaCtrl::EnableScrollBars(int nBar, BOOL bEnable /*=TRUE*/)
{
    if (bEnable)
    {
        if (!IsVisibleHScroll() && (nBar == SB_HORZ || nBar == SB_BOTH))
        {
            CWnd::EnableScrollBarCtrl(SB_HORZ, bEnable);
            m_nBarState |= FCSB_HORZ;
        }
        
        if (!IsVisibleVScroll() && (nBar == SB_VERT || nBar == SB_BOTH))
        {
            CWnd::EnableScrollBarCtrl(SB_VERT, bEnable);
            m_nBarState |= FCSB_VERT;
       }
    }
    else
    {
        if ( IsVisibleHScroll() && (nBar == SB_HORZ || nBar == SB_BOTH))
        {
            CWnd::EnableScrollBarCtrl(SB_HORZ, bEnable);
            m_nBarState &= ~FCSB_HORZ; 
        }
        
        if ( IsVisibleVScroll() && (nBar == SB_VERT || nBar == SB_BOTH))
        {
            CWnd::EnableScrollBarCtrl(SB_VERT, bEnable);
            m_nBarState &= ~FCSB_VERT;
        }
    }
}

// Get/Set scroll position using 32 bit functions
int CFormulaCtrl::GetScrollPos32(int nBar, BOOL bGetTrackPos /* = FALSE */)
{
    SCROLLINFO si;
    si.cbSize = sizeof(SCROLLINFO);

    if (bGetTrackPos)
    {
        if (GetScrollInfo(nBar, &si, SIF_TRACKPOS))
            return si.nTrackPos;
    }
    else
    {
        if (GetScrollInfo(nBar, &si, SIF_POS))
            return si.nPos;
    }

    return 0;
}


BOOL CFormulaCtrl::SetScrollPos32(int nBar, int nPos, BOOL bRedraw /* = TRUE */)
{
    SCROLLINFO si;
    si.cbSize = sizeof(SCROLLINFO);
    si.fMask  = SIF_POS;
    si.nPos   = nPos;
    return SetScrollInfo(nBar, &si, bRedraw);
}

// Handle horz scrollbar notifications
void CFormulaCtrl::OnHScroll(UINT nSBCode, UINT /*nPos*/, CScrollBar* /*pScrollBar*/)
{
	int scrollPos = GetScrollPos32(SB_HORZ);

	CRect rect;
	GetClientRect(rect);

	switch (nSBCode)
	{
	case SB_LINERIGHT:
		{
			SetScrollPos32(SB_HORZ, (scrollPos<m_nHScrollMax-10) ? (scrollPos+10) :m_nHScrollMax );
			Invalidate();
		}
		break;
	case SB_PAGERIGHT:
		{
			SetScrollPos32(SB_HORZ, (scrollPos<m_nHScrollMax-rect.Width()) ? (scrollPos+rect.Width()) :m_nHScrollMax );
			Invalidate();
		}
		break;
	case SB_LINELEFT:
		{
			SetScrollPos32(SB_HORZ, (scrollPos>10) ? (scrollPos-10) :0 );
			Invalidate();
		}
		break;        
	case SB_PAGELEFT:
		{
			SetScrollPos32(SB_HORZ, (scrollPos>rect.Width()) ? (scrollPos-rect.Width()) :0 );
			Invalidate();
		}
		break;        
	case SB_THUMBPOSITION:
	case SB_THUMBTRACK:
		{
			SetScrollPos32(SB_HORZ, GetScrollPos32(SB_HORZ, TRUE));
			Invalidate();
		}
		break;

	case SB_LEFT:
		if (scrollPos > 0)
		{
			SetScrollPos32(SB_HORZ, 0);
			Invalidate();
		}
		break;

	case SB_RIGHT:
		if (scrollPos < m_nHScrollMax)
		{
			SetScrollPos32(SB_HORZ, m_nHScrollMax);
			Invalidate();
		}
		break;

	default: 
		break;
	}
}

// Handle vert scrollbar notifications
void CFormulaCtrl::OnVScroll(UINT nSBCode, UINT /*nPos*/, CScrollBar* /*pScrollBar*/)
{
	int scrollPos = GetScrollPos32(SB_VERT);

	CRect rect;
	GetClientRect(rect);

	switch (nSBCode)
	{
	case SB_LINEDOWN:
		{
			SetScrollPos32(SB_VERT, (scrollPos<m_nVScrollMax-10) ? (scrollPos+10) : m_nVScrollMax );
			Invalidate();
		}
		break;
	case SB_PAGEDOWN:
		{
			SetScrollPos32(SB_VERT, (scrollPos<m_nVScrollMax-rect.Height()) ? (scrollPos+rect.Height()) : m_nVScrollMax );
			Invalidate();
		}
		break;
	case SB_LINEUP:
		{
			SetScrollPos32(SB_VERT, (scrollPos>10) ? (scrollPos-10) :0 );
			Invalidate();
		}
		break;
	case SB_PAGEUP:
		{
			SetScrollPos32(SB_VERT, (scrollPos>rect.Height()) ? (scrollPos-rect.Height()) :0 );
			Invalidate();
		}
		break;
	case SB_THUMBPOSITION:
	case SB_THUMBTRACK:
		{
			SetScrollPos32(SB_VERT, GetScrollPos32(SB_VERT, TRUE));
			Invalidate();
		}
		break;

	case SB_TOP:
		if (scrollPos > 0)
		{
			SetScrollPos32(SB_VERT, 0);
			Invalidate();
		}
		break;

	case SB_BOTTOM:
		if (scrollPos < m_nVScrollMax)
		{
			SetScrollPos32(SB_VERT, m_nVScrollMax);
			Invalidate();
		}

	default: 
		break;
	}
}

CRect CFormulaCtrl::GetVirtualRect()
{
	return m_VirtualRect;
}

// If resizing or changes occur, call this - it'll fix up the scroll bars
void CFormulaCtrl::ResetScrollBars()
{
    if (!::IsWindow(GetSafeHwnd())) 
        return;
    
    CRect rect;
    GetClientRect(rect);
    
    if (rect.left == rect.right || rect.top == rect.bottom)
        return;
    
    if (IsVisibleVScroll())
        rect.right += GetSystemMetrics(SM_CXVSCROLL) + GetSystemMetrics(SM_CXBORDER);
    
    if (IsVisibleHScroll())
        rect.bottom += GetSystemMetrics(SM_CYHSCROLL) + GetSystemMetrics(SM_CYBORDER);
    
    if (rect.left >= rect.right || rect.top >= rect.bottom)
    {
        EnableScrollBarCtrl(SB_BOTH, FALSE);
        return;
    }

    CRect VisibleRect(0, 0, rect.right, rect.bottom);
    CRect VirtualRect(GetVirtualRect());
    
    // If vertical scroll bar, horizontal space is reduced
    if (VisibleRect.Height() < VirtualRect.Height())
        VisibleRect.right -= ::GetSystemMetrics(SM_CXVSCROLL);
    // If horz scroll bar, vert space is reduced
    if (VisibleRect.Width() < VirtualRect.Width())
        VisibleRect.bottom -= ::GetSystemMetrics(SM_CYHSCROLL);
    
    // Recheck vertical scroll bar
    if (VisibleRect.Height() < VirtualRect.Height())
    {
        EnableScrollBars(SB_VERT, TRUE); 
        m_nVScrollMax = VirtualRect.Height() - 1;
    }
    else
    {
        EnableScrollBars(SB_VERT, FALSE); 
        m_nVScrollMax = 0;
    }

    if (VisibleRect.Width() < VirtualRect.Width())
    {
        EnableScrollBars(SB_HORZ, TRUE); 
        m_nHScrollMax = VirtualRect.Width() - 1;
    }
    else
    {
        EnableScrollBars(SB_HORZ, FALSE); 
        m_nHScrollMax = 0;
    }

    ASSERT(m_nVScrollMax < INT_MAX && m_nHScrollMax < INT_MAX); // This should be fine

    SCROLLINFO si;
    si.cbSize = sizeof(SCROLLINFO);
    si.fMask = SIF_PAGE;
    si.nPage = (m_nHScrollMax>0)? VisibleRect.Width() : 0;
    SetScrollInfo(SB_HORZ, &si, FALSE); 
    si.nPage = (m_nVScrollMax>0)? VisibleRect.Height() : 0;
    SetScrollInfo(SB_VERT, &si, FALSE);

    SetScrollRange(SB_VERT, 0, m_nVScrollMax, TRUE);
    SetScrollRange(SB_HORZ, 0, m_nHScrollMax, TRUE);
}

void CFormulaCtrl::OnSize(UINT nType, int nCx, int nCy) 
{
    if (!::IsWindow(m_hWnd))
        return;
    m_DropTarget.Register(this);

	CWnd::OnSize(nType, nCx, nCy);
    ResetScrollBars();	
}



BOOL CFormulaCtrl::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
    // A m_nRowsPerWheelNotch value less than 0 indicates that the mouse
    // wheel scrolls whole pages, not just lines.
    if (m_nRowsPerWheelNotch == -1)
    {
        int nPagesScrolled = zDelta / 120;

        if (nPagesScrolled > 0)
            for (int n = 0; n < nPagesScrolled; n++)
                PostMessage(WM_VSCROLL, SB_PAGEUP, 0);
        else
            for (int n = 0; n > nPagesScrolled; n--)
                PostMessage(WM_VSCROLL, SB_PAGEDOWN, 0);
    }
    else
    {
        int nRowsScrolled = m_nRowsPerWheelNotch * zDelta / 120;

        if (nRowsScrolled > 0)
            for (int i = 0; i < nRowsScrolled; i++)
                PostMessage(WM_VSCROLL, SB_LINEUP, 0);
        else
            for (int i = 0; i > nRowsScrolled; i--)
                PostMessage(WM_VSCROLL, SB_LINEDOWN, 0);
    }
	return CWnd::OnMouseWheel(nFlags, zDelta, pt);
}

void CFormulaCtrl::OnEndInPlaceEdit(NMHDR* pNMHDR, LRESULT* pResult)
{
      // Send Notification to parent
	SendMessageToParent(LVN_ENDLABELEDIT);
	m_pEditWnd=NULL;
	GetBinTree()->SetNodeSelectionType(NS_NODE);
	GetBinTree()->CheckPoint();
	Invalidate(FALSE);
}

// Sends a message to the parent in the form of a WM_NOTIFY message with
// a NMHDR structure attached
LRESULT CFormulaCtrl::SendMessageToParent(int nMessage) const
{
    if (!IsWindow(m_hWnd))
        return 0;
	NMHDR hdr;

    hdr.hwndFrom = m_hWnd;
    hdr.idFrom   = GetDlgCtrlID();
    hdr.code     = nMessage;

    CWnd *pOwner = GetOwner();
    if (pOwner && IsWindow(pOwner->m_hWnd))
        return pOwner->SendMessage(WM_NOTIFY, hdr.idFrom, (LPARAM)&hdr);
    else
        return 0;
}
