/////////////////////////////////////////////////////////////////////////////
//  File:       nodeentities.h
//  Version:    1.0.0.0
//  Created:    03-april-2002
//
//  Author:     Thorsten Wack
//  E-mail:     wt@umsicht.fhg.de
//
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. If 
// the source code in  this file is used in any commercial application 
// then a simple email would be nice.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability if it causes any damage whatsoever.
//
/////////////////////////////////////////////////////////////////////////////

#ifndef _ENTITY_H
#define _ENTITY_H

#include "node.h"


////////////////////////////////////////////////////////////////////
class CTypedNode : public CNode
{
public:
	CTypedNode();
	virtual ~CTypedNode();

	virtual const void operator=(const CNode& Node);

	void SetType(int nType);
	int GetType();
	virtual void Serialize(CArchive& ar, CBinTree* pTree);
	virtual CString WriteNode(int nLanguageType);

protected:
	CStringArray m_straShortCuts;
	CStringArray m_straFontNames;
	int m_nType;
	CStringArray m_straKeyWords[LT_COUNT];
	CStringArray m_straCodeFormats[LT_COUNT];
};

class CValueNode : public CNode
{
public:
	CValueNode();
	virtual ~CValueNode();

	virtual const void operator=(const CNode& Node);

	void SetValue(double dValue);
	double GetValue();

	void SetUnit(long lIDUnit);
	long GetUnit();
	
	void SetContext(long lIDContext);
	long GetContext();

	virtual void FormatNode(CBinTreeFormat* pFormat=NULL);
	virtual void Serialize(CArchive& ar, CBinTree* pTree);
protected:
	double m_dValue;
	long m_lIDContext;
	long m_lIDUnit;
};
////////////////////////////////////////////////////////////////////

class CVariableNode : public CValueNode
{
public:
	CVariableNode();
	virtual ~CVariableNode();
};

class CConstantNode : public CValueNode
{
public:
	CConstantNode();
	virtual ~CConstantNode();
};

class CNumberNode : public CValueNode
{
public:
	CNumberNode();
	virtual ~CNumberNode();
};

class COpNode : public CTypedNode
{
public:
	COpNode();
	virtual ~COpNode();
};

class CFunctionNode : public CTypedNode
{
public:
	CFunctionNode();
	virtual ~CFunctionNode();
};

class CExtFuncNode : public CFunctionNode
{
public:
	CExtFuncNode();
	virtual ~CExtFuncNode();
};

class CUserFuncNode : public CFunctionNode
{
public:
	CUserFuncNode();
	virtual ~CUserFuncNode();
};


class CDivisionNode : public COpNode
{
public:
	CDivisionNode();
	virtual ~CDivisionNode();

	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CNthRootNode : public CNode
{
public:
	CNthRootNode();
	virtual ~CNthRootNode();

	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CSquareRootNode : public CNthRootNode
{
public:
	CSquareRootNode();
	virtual ~CSquareRootNode();
};

class CPowerToNode : public CNode
{
public:
	CPowerToNode();
	virtual ~CPowerToNode();

	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CPlaceHolderNode : public CNode
{
public:
	CPlaceHolderNode();
	virtual ~CPlaceHolderNode();
	
	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CBraceNode : public COpNode
{
public:
	CBraceNode();
	virtual ~CBraceNode();

	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CEquationNode : public COpNode
{
public:
	CEquationNode();
	virtual ~CEquationNode();

	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CPartialDerivativeNode : public CNode
{
public:
	CPartialDerivativeNode();
	virtual ~CPartialDerivativeNode();

	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CDerivativeNode : public CPartialDerivativeNode
{
public:
	CDerivativeNode();
	virtual ~CDerivativeNode();
};

class CSumNode : public CNode
{
public:
	CSumNode();
	virtual ~CSumNode();
	virtual void Serialize(CArchive& ar, CBinTree* pTree);

//	virtual CNode* GetNodeFromPoint(CDC* pDC, CRect& rect, CPoint point);
	virtual void FormatNode(CBinTreeFormat* pFormat=NULL);
	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CProdNode : public CSumNode
{
public:
	CProdNode();
	virtual ~CProdNode();
};

class CRangeNode : public CNode
{
protected:
	int m_nRangeType;
public:
	CRangeNode(CNode* pParent=NULL);
	virtual ~CRangeNode();

	virtual CNode* GetNodeFromPoint(CDC* pDC, CRect& rect, CPoint point);
	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CIntegrandNode: public CNode
{
public:
	CIntegrandNode();
	virtual ~CIntegrandNode();
};

class CIntegralNode : public COpNode
{
public:
	CIntegralNode();
	virtual ~CIntegralNode();
	
//	virtual CNode* GetNodeFromPoint(CDC* pDC, CRect& rect, CPoint point);
	virtual void FormatNode(CBinTreeFormat* pFormat=NULL);
	virtual void Serialize(CArchive& ar, CBinTree* pTree);
	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CFuncNode : public CFunctionNode
{
public:
	CFuncNode();
	virtual ~CFuncNode();

	virtual void FormatNode(CBinTreeFormat* pFormat=NULL);
	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CPoisonNode : public CBraceNode
{
public:
	CPoisonNode();
	virtual ~CPoisonNode();

	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CIndexNode : public CNode
{
public:
	CIndexNode(CNode* pParent=NULL);
	virtual ~CIndexNode();

	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CVectorNode : public COpNode
{
public:
	CVectorNode();
	virtual ~CVectorNode();

	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CLimesNode : public CNode
{
public:
	CLimesNode();
	virtual ~CLimesNode();

	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class COperatorNode : public COpNode
{
public:
	COperatorNode();
	virtual ~COperatorNode();
};

class CArrowNode: public COpNode
{
public:
	CArrowNode();
	virtual ~CArrowNode();

	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CBinomialNode : public CNode
{
public:
	CBinomialNode();
	virtual ~CBinomialNode();

	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CMatrixNode : public CNode
{
protected:
	int m_nCols;
	int m_nRows;
public:
	CMatrixNode();
	virtual ~CMatrixNode();

	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CLineNode : public CNode
{
protected:
	int m_nCol;
	int m_nRow;
public:
	CLineNode();
	virtual ~CLineNode();

	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CElementNode : public CNode
{
protected:
	int m_nCol; 
public:
	CElementNode();
	virtual ~CElementNode();

	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CNablaNode : public CNode
{
public:
	CNablaNode();
	virtual ~CNablaNode();

	virtual void TransformRects(CRect& ,CRect& ,CRect& ,CRect& );
	virtual CRect GetContentRect(CDC* pDC);
	virtual void DrawContent(CDC* pDC, CRect rect, DWORD dwSelect);
};

class CLaplaceNode : public CNode
{
public:
	CLaplaceNode();
	virtual ~CLaplaceNode();
};

class CInfinityNode : public CConstantNode 
{
public:
	CInfinityNode();
	virtual ~CInfinityNode();
};

class CPlanckNode : public CConstantNode 
{
public:
	CPlanckNode();
	virtual ~CPlanckNode();
};

class CLambdaNode : public CConstantNode 
{
public:
	CLambdaNode();
	virtual ~CLambdaNode();
};

class CPlusNode : public COpNode
{
public:
	CPlusNode();
	virtual ~CPlusNode();
};

class CMinusNode : public COpNode
{
public:
	CMinusNode();
	virtual ~CMinusNode();
};

class CTimesNode : public COpNode
{
public:
	CTimesNode();
	virtual ~CTimesNode();
};

class CCrossNode : public COpNode
{
public:
	CCrossNode();
	virtual ~CCrossNode();
};

class CLabelNode : public CNode
{
public:
	CLabelNode();
	virtual ~CLabelNode();
};

#endif
